#!/usr/bin/python
# -*- coding: utf-8 -*-

##Copyright (c) 2017 Benoit Valot and Panisa Treepong
##benoit.valot@univ-fcomte.fr
##UMR 6249 Chrono-Environnement, Besançon, France
##Licence GPL

"""Read Bam File and store clip read by position"""

from pysam import AlignmentFile
from .clipread import ClipRead
from .position import Position, Positions
from . import variables

def parse(bamfile, minqual):
    bamhandle =  AlignmentFile(bamfile, 'rb')
    
    positions = Positions()
    
    for read in bamhandle:
        if isclip(read) is False or read.mapping_quality < minqual:
            continue
        clip = ClipRead(read)
        pos = positions.getposition(bamhandle.get_reference_name(read.reference_id), \
                                    clip.getclippos())
        pos.addclipread(clip)
        
    bamhandle.close()    
    return positions


def isclip(read):
    """Look at if the read is a clip read"""
    ##return False for unmapped, or full map
    if read.is_unmapped or read.cigartuples is None or len(read.cigartuples) == 1:
        return False
    ##search cigar value in start and end
    clip_start = read.cigartuples[0][0] in variables.cigarclip
    clip_end = read.cigartuples[-1][0] in variables.cigarclip
    if clip_start :
        if clip_end:
            return False ##clip read at both end not take into account
        else:
            return True
    elif clip_end:
        return True
    else:
        return False

if __name__=='__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/python
# -*- coding: utf-8 -*-

##Copyright (c) 2017 Benoit Valot and Panisa Treepong
##benoit.valot@univ-fcomte.fr
##UMR 6249 Chrono-Environnement, Besançon, France
##Licence GPL

from . import invertedrepeat as ir

class Couple():
    """Class containing a start and end clip position with potential IS"""
    def __init__(self, chrom, posstart, posend):
        self.chrom = chrom
        self.posstart = posstart
        self.posend = posend
        self.cons5prime = None
        self.cons3prime = None
        self.dr = None
        self.ir = None
        
    def createconsensus(self, percent):
        """Create consensus of clip read at both side"""
        self.cons3prime = self.posstart.getconsensus(percent, True)
        self.cons5prime = self.posend.getconsensus(percent, False)
        self.dr = self.__createdr(percent)
        
    def searchir(self):
        if self.cons5prime is None or self.cons3prime is None:
            raise Exception("You must create consensus before search ir")
        self.ir = ir.searchir(self.cons5prime, self.cons3prime)

    def rangeposition(self):
        return range(self.posstart.pos, self.posend.pos+1)

    def __createdr(self, percent):
        size = self.posend.pos - self.posstart.pos
        if size == 0:
            return ""
        seqs = []
        for clip in self.posstart.clipstart:
            s = clip.getdr(self.posstart.pos, self.posend.pos)
            if s is not None and len(s) == self.posend.pos - self.posstart.pos:
                seqs.append(s)
        for clip in self.posend.clipend:
            s = clip.getdr(self.posstart.pos, self.posend.pos)
            if s is not None and len(s) == self.posend.pos - self.posstart.pos:
                seqs.append(s)
        if len(seqs) == 0 :
            return "No sequence to build direct repeat"
        if len(set(map(len,seqs))) != 1:
            raise Exception("Direct repeat region can only compute on same length")
        dr_tmp = []
        for i in range(0,len(seqs[0])):
            allele = set()
            values = []
            for seq in seqs:
                base = seq[i]
                allele.add(base)
                values.append(base)
            toadd = "N"
            for base in allele:
                if len(values) >= 2 and \
                   float(values.count(base))/len(values) >= percent:
                    toadd = base
            dr_tmp.append(toadd)
        return "".join(dr_tmp)
    
    def __str__(self):
        toreturn = str(self.posstart) + "\t" + str(self.posend) + "\t" + self.dr + "\t"
        if self.ir is None:
            toreturn += "No IR"
        else:
            toreturn += str(self.ir)
        toreturn += "\t" + self.cons5prime + "\t" + self.cons3prime
        return  toreturn
        
    def __len__(self):
        return self.posend.pos - self.posstart.pos

    def __repr__(self):
        return self.posstart + " - " + self.posend


class Couples():
    """Class containing all the couple"""
    def __init__(self):
        self.couples = []

    def groupeposition(self, positions, size):
        """Fonction to parse the positions 
        and regroup start and end clip positions to a couple"""
        posstart = []
        posend = []
        chrom = None
        for pos in positions.nextposition():
            if chrom is None: ##first position
                chrom = pos.chrom
            elif chrom != pos.chrom: ##change chromosome, create couple and reset
                self.__createcouples(chrom, posstart, posend, size)
                posstart = []
                posend = []
                chrom = pos.chrom
            ##add position to list
            if len(pos.clipstart) != 0:
                posstart.append(pos)
            if len(pos.clipend) != 0:
                posend.append(pos)
        ##add last chromomose couple
        self.__createcouples(chrom, posstart, posend, size)

    
    def __createcouples(self, chrom, posstart, posend, size):
        """Create all couple from same chromomosome"""
        for s in posstart:
            if s.chrom != chrom:
                raise Exception("Only position with same chromosome can be coupled")
            for e in posend:
                if e.chrom != chrom:
                    raise Exception("Only position with same chromosome can be coupled")
                diff = (e.pos - s.pos)
                if diff >= 0 and diff <= size:
                    self.couples.append(Couple(chrom,s,e))

    def filteroverlapcouple(self):
        """Function to filter overlap couples and return best couple"""
        couplestmp = []
        cur_couple = None
        for couple in self.couples:
            if cur_couple is None:
                cur_couple = couple
            else:
                if self.__overlap(couple, cur_couple):
                    cur_couple = self.__best(couple, cur_couple)
                else:
                    couplestmp.append(cur_couple)
                    cur_couple = couple
        if cur_couple is not None:
            couplestmp.append(cur_couple)
        self.couples = couplestmp

    def __overlap(self, couple1, couple2):
        if couple1.chrom != couple2.chrom:
            return False
        else:
            a = set(couple1.rangeposition())
            b = set(couple2.rangeposition())
            if len(a.intersection(b)) == 0:
                return False
            else:
                return True

    def __best(self, couple1, couple2):
        diff = len(couple1.posstart.clipstart) - len(couple2.posstart.clipstart) + \
               len(couple1.posend.clipend) - len(couple2.posend.clipend)
        if diff < 0:
            return couple2
        elif diff > 0:
            return couple1
        else:
            if len(couple1) > len(couple2):
                return couple1
            else:
                return couple2
    
    def __iter__(self):
        for couple in self.couples:
            yield couple

if __name__=='__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/python
# -*- coding: utf-8 -*-

##Copyright (c) 2017 Benoit Valot and Panisa Treepong
##benoit.valot@univ-fcomte.fr
##UMR 6249 Chrono-Environnement, Besançon, France
##Licence GPL

import subprocess
from . import temporalfile as temp

class InvertRepeat():
    """Class containing inverted repeat at the 5' and the 3'"""
    def __init__(self, prime5_pos, prime5_seq, prime3_pos, prime3_seq):
        ##position beginning ir from clip end consensus (1)
        self.prime5_pos = prime5_pos
        ##Sequence ir from clip end consensus
        self.prime5_seq = prime5_seq
        ##position - len(seq) -1 beginning ir from clip start consensus (-1)
        self.prime3_pos = prime3_pos
        ##sequence ir from clip start consensus (reverse)
        self.prime3_seq = prime3_seq

    def __str__(self):
        return " ".join([str(self.prime5_pos), self.prime5_seq.upper(), "--", \
                         self.prime3_seq.upper(), str(self.prime3_pos)])

    def isinrange(self, range_ir):
        if self.prime5_pos <= range_ir and self.prime3_pos >= -range_ir :
            return True

    def isbestir(self, other_ir):
        diffa = (self.prime5_pos - self.prime3_pos)
        diffb = (other_ir.prime5_pos - other_ir.prime3_pos)
        if diffa < diffb:
            return False
        elif diffa > diffb:
            return True
        else:
            raise Exception("Implement differentiation with similar IR pos")
        

def searchir(prime5seq, prime3seq):
    """Searched ir from 5 to 3 prime consensus and return InvertRepeat class or None"""
    range_ir_pos = 15

    ##test if consensus sequence is not empty
    if prime5seq == "" or prime3seq:
        return None
    
    outseq_data = __performedirsearchwitheinverted(prime5seq, prime3seq)
    len_cons = __lenofconsensus(prime5seq, prime3seq)

    for invert in __getnextirfromeinverted(outseq_data,len_cons):
        if invert is None:
            return None
        invertRepeat = None

        ##Check IR in range and choice best one
        if invert.isinrange(range_ir_pos):
            if invertRepeat is None or invert.isbestir(invertRepeat):
                invertRepeat = str(invert)
                
    return invertRepeat


def __performedirsearchwitheinverted(prime5seq, prime3seq):
    """Take 5 and 3 prime consensus to search invert repeat with einverted"""
    intempfile = temp.writefile("\n".join([">",prime5seq+prime3seq]))

    outtempfile = temp.createfile()
    seqtempfile = temp.createfile()

    ##Call einverted from emboss to find IR
    commandir = ['einverted -sequence '+ intempfile.name +' -gap 12 -threshold 15 -match 3 -mismatch -4 -outfile ' \
    + outtempfile.name + ' -outseq '+ seqtempfile.name + ' -auto Y -warning N']
    processir = subprocess.Popen(commandir, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    processir.communicate()

    ##Read data from outseq of einverted
    outseq_data = temp.readfile(seqtempfile,">")


    ##Close all temp files
    temp.closefile(intempfile)
    temp.closefile(outtempfile)
    temp.closefile(seqtempfile)    

    return outseq_data


def __getnextirfromeinverted(outseq_data,len_cons):
    """read the fasta export of einverted to return invertrepeat object"""
    if len(outseq_data) < 1: ##No IR
        yield None
    
    for block in range(int(len(outseq_data)/2)):#a block refers a result set that contains 2 lists
        line = 2*block
        ##create IR
        prime5_pos_t = int(outseq_data[0+line].split("\n")[0].split('_')[-2])
        prime5_seq_t = "".join(outseq_data[0+line].split("\n")[1:])
        prime3_pos_t = int(outseq_data[1+line].split("\n")[0].split('_')[-1])-len_cons-1
        prime3_seq_t = "".join(outseq_data[1+line].split("\n")[1:])
        invert = InvertRepeat(prime5_pos_t, prime5_seq_t, prime3_pos_t, prime3_seq_t)
        yield invert

def __lenofconsensus(prime5seq, prime3seq):
    return len(prime5seq+prime3seq)

if __name__=='__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/python
# -*- coding: utf-8 -*-

##Copyright (c) 2017 Benoit Valot and Panisa Treepong
##benoit.valot@univ-fcomte.fr
##UMR 6249 Chrono-Environnement, Besançon, France
##Licence GPL

"""Search integrative element insertion on BAM alignment"""

import sys
import argparse
import os
import tempfile
from lib import bamreader
from lib.couple  import Couples
from lib.writer import writetabular

desc = "Search integrative element (IS) insertion on a genome using BAM alignment"
command = argparse.ArgumentParser(prog='panISa.py', \
    description=desc, usage='%(prog)s [options] bam')
command.add_argument('-o', '--output', nargs="?", \
    type=argparse.FileType("w"), default=sys.stdout, \
    help='Return list of IS insertion by alignment, default=stdout')
command.add_argument('-q', '--quality', nargs="?", \
    type=int, default=20, \
    help='Min alignment quality value to conserve a clipped read, default=20')
command.add_argument('-m', '--minimun', nargs="?", \
    type=int, default=10, \
    help='Min number of clipped reads to look at IS on a position, default=10')
command.add_argument('-s', '--size', nargs="?", \
    type=int, default=20, \
    help='Maximun size of direct repeat region, default=20pb')
command.add_argument('-p', '--percentage', nargs="?", \
    type=float, default=0.8, \
    help='Minimum percentage of same base to create consensus, default=0.8')
command.add_argument('bam', type=argparse.FileType("r"), \
    help='Alignment on BAM/SAM format')
command.add_argument('-v', '--version', action='version', \
    version='%(prog)s 0.1.2')


if __name__=='__main__':
    """Performed job on execution script""" 
    args = command.parse_args()
    output = args.output
    ##valided argument
    if args.percentage < 0 or args.percentage > 1:
        raise Exception("--percentage must be comprised between 0 to 1")
    ##Search clip read on bam by position with min quality
    positions = bamreader.parse(args.bam.name, args.quality)

    ##filter positions with not enought clipread
    positions.filterposition(args.minimun)
        
    ##find close position with both start and end clip to make a couple
    couples = Couples()
    couples.groupeposition(positions, args.size)
    couples.filteroverlapcouple()
        
    ##Create consensus and search inverted repeat sequence on it
    for couple in couples:
        couple.createconsensus(args.percentage)
        couple.searchir()
    
    ##return list of possible IS
    writetabular(output, couples)



#!/usr/bin/python
# -*- coding: utf-8 -*-

##Copyright (c) 2017 Benoit Valot and Panisa Treepong
##benoit.valot@univ-fcomte.fr
##UMR 6249 Chrono-Environnement, Besançon, France
##Licence GPL

from . import variables

class ClipRead():
    """Clip read object"""
    def __init__(self, alignedsegment):
        self.read_seq = alignedsegment.query_sequence
        self.read_name = alignedsegment.query_name
        self.read_start = alignedsegment.query_alignment_start #0 left
        self.read_end = alignedsegment.query_alignment_end #exclusive
        self.read_len = alignedsegment.query_alignment_length
        self.ref_start = alignedsegment.reference_start #0 left 
        self.ref_end = alignedsegment.reference_end # exclusive
        self.ref_len = alignedsegment.reference_length
        self.cigar = alignedsegment.cigarstring
        self.cigartuples = alignedsegment.cigartuples
        self.isreverse = alignedsegment.is_reverse
        
    def isstartclip(self):
        """Test if the read is start or end clip, look at """
        if self.cigartuples is None:
            raise Exception("ClipRead must be aligned")
        if self.cigartuples[0][0] in variables.cigarclip:
            return True
        elif self.cigartuples[-1][0] in variables.cigarclip:
            return False
        else:
            raise Exception("ClipRead must contain clip part at start or end")

    def getdr(self, drstart, drend):
        """Return the dr sequence if complete or return None"""
        s = self.read_start + (drstart - self.ref_start) ##if < 0, incomplet dr
        if s < 0:
            return None       
        e = self.read_end - (self.ref_end - drend)
        if e > len(self.read_seq):
            return None
        return self.read_seq[s:e]

    def getclippos(self):
        """Return the position of the clip"""
        if self.isstartclip():
            return self.ref_start
        else:
            return self.ref_end

    def getclipseq(self):
        """return clip part of the read, except for hard clip return None"""
        if len(self.read_seq) == self.read_len:
            return None
        if self.isstartclip():
            return self.read_seq[:self.read_start]
        else:
            return self.read_seq[self.read_end:]
        
    def __len__(self):
        return len(self.read_seq)

    def __repr__(self):
        return self.read_seq

    def __str__(self):
        return str(self.ref_start) + ": " + str(self.read_start) + self.read_seq + \
            str(self.read_end) + " :" + str(self.ref_end)

if __name__=='__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/python
# -*- coding: utf-8 -*-

##Copyright (c) 2017 Benoit Valot and Panisa Treepong
##benoit.valot@univ-fcomte.fr
##UMR 6249 Chrono-Environnement, Besançon, France
##Licence GPL


import tempfile
import traceback


def createfile():
    newtempfile = tempfile.NamedTemporaryFile('w+t')
    return newtempfile

        
def closefile(closefile):
    closefile.close()


def writefile(writedata):
    temp = tempfile.NamedTemporaryFile('w+t')
    try:
        temp.write(writedata)
        temp.seek(0)    
    except Exception:
        exe = traceback.format_exc()
        print(exe)
    return temp


def readfile(tempread,separateBy):
    readdata = []
    sep_line = ["\n","line"]
    try:
        tempread.seek(0)
        if separateBy in sep_line:
            readdata = tempread.readlines()
        else:
            readdata = tempread.read().split(separateBy)[1:]
    except Exception:
        exe = traceback.format_exc()
        print(exe)
    return readdata


if __name__=='__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/python
# -*- coding: utf-8 -*-

##Copyright (c) 2017 Benoit Valot and Panisa Treepong
##benoit.valot@univ-fcomte.fr
##UMR 6249 Chrono-Environnement, Besançon, France
##Licence GPL

def writetabular(output,couples):
    output.write("\t".join(["Chromosome","End position", "End clipped reads", "Direct repeats","Start position", "Start clipped reads", \
        "Inverted repeats","Left sequence","Right sequence"]) + "\n")
    for c in couples:
        towrite = []
        towrite.extend([c.chrom, str(c.posend.pos), str(len(c.posend.clipend)), \
                        c.dr, str(c.posstart.pos + 1), str(len(c.posstart.clipstart))])
        if c.ir is None:
            towrite.append("No IR")
        else:
            towrite.append(c.ir)
        towrite.extend([c.cons5prime, c.cons3prime])

        output.write("\t".join(towrite) + "\n")

if __name__=='__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/python
# -*- coding: utf-8 -*-

##Copyright (c) 2017 Benoit Valot and Panisa Treepong
##benoit.valot@univ-fcomte.fr
##UMR 6249 Chrono-Environnement, Besançon, France
##Licence GPL


class Position():
    """Class that store clip read at a genome position"""
    def __init__(self, chrom, pos):
        self.chrom = chrom
        self.pos = pos
        self.clipstart = []
        self.clipend = []

    def addclipread(self, clipread):
        if clipread.isstartclip():
            self.clipstart.append(clipread)
        else:
            self.clipend.append(clipread)

    def removeclipside(self, minread):
        if len(self.clipstart) < minread:
            self.clipstart = []
        if len(self.clipend) < minread:
            self.clipend = []
            
    def getconsensus(self, percent, start):
        if start:
            seqs = [clip.getclipseq() for clip in self.clipstart]
        else:
            seqs = [clip.getclipseq() for clip in self.clipend]

        while None in seqs:
            seqs.remove(None)
        if len(seqs) == 0:
            return ""
        cons = []
        for i in range(0, max(map(len, seqs))):
            allele = set()
            values = []
            for seq in seqs:
                if len(seq) <= i:
                    continue
                if start:
                    base = seq[-i-1]
                else:
                    base = seq[i]
                allele.add(base)
                values.append(base)
            toadd = "N"
            for base in allele:
                if len(values) >= 2 and \
                   float(values.count(base))/len(values) >= percent:
                    toadd = base
            cons.append(toadd)
        if start:
            return "".join(cons[::-1])
        else:
            return "".join(cons)
    
    def __len__(self):
        return max(len(self.clipstart), len(self.clipend))
        
    def __repr__(self):
    	return self.chrom + " : " + str(self.pos)

    def __str__(self):
        return "\t".join([self.chrom, str(self.pos), str(len(self.clipstart)), \
                          str(len(self.clipend))])
    def __eq__(self, other):
        if self.chrom == other.chrom and self.pos == other.pos:
            return True
        else:
            return False

    def __lt__(self, other):
        if self.chrom == other.chrom:
            return self.pos < other.pos
        else:
            return self.chrom < other.chrom
        

class Positions():
    """Container of all positions with a clip read"""
    def __init__(self):
        self.posdict = {}

    def getposition(self, chrom, pos):
        return self.posdict.setdefault(chrom+"_"+str(pos), Position(chrom, pos))

    def filterposition(self, minread):
        """Filter the positions to remove position insufficiant minread"""
        tmpdict = {}
        for item in self.posdict.items():
            if len(item[1]) >= minread:
                item[1].removeclipside(minread)
                tmpdict[item[0]] = item[1]
        self.posdict = tmpdict

    def nextposition(self):
        """Create iterator that return position in the order of chromosome"""
        posits = list(self.posdict.values())
        posits.sort()
        for pos in posits:
            yield pos
        
                
if __name__=='__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/python
# -*- coding: utf-8 -*-

##Copyright (c) 2017 Benoit Valot and Charlotte Couchoud
##benoit.valot@univ-fcomte.fr
##UMR 6249 Chrono-Environnement, Besançon, France
##Licence GPL

"""automate search in ISFinder from panISa output"""

import argparse
import sys
import requests
try:
    from HTMLParser import HTMLParser
except ImportError:
    from html.parser import HTMLParser

desc = "automate search IS homology in ISFinder from panISa output"
command = argparse.ArgumentParser(prog='ISFinder_search.py', \
    description=desc, usage='%(prog)s [options] file')
command.add_argument('-o', '--output', nargs="?", \
    type=argparse.FileType("wt"), default=sys.stdout, \
    help='Return potential IS found in ISFinder (default:stdout)')
command.add_argument('-r', '--remove', nargs="?", \
    type=str, default=".txt", \
    help='Remove suffix at the end of the filename (default:.txt)')
command.add_argument('-l', '--length', nargs="?", \
    type=int, default=30, \
    help='Length of the IRR-IRL search (default:30)')
command.add_argument('-i', '--identity', nargs="?", \
    type=int, default=90, choices=range(0,101), metavar="0-100", \
    help='Percentage of expected identity (default:90)')
command.add_argument('-e', '--evalue', nargs="?", \
    type=float, default=0.001, \
    help='Expected max evalue (default:1E-3)')
command.add_argument('-a', '--alignment', nargs="?", \
    type=float, default=80, choices=range(0,101), metavar="0-100", \
    help='Percentage of expected alignment (default:80)')
command.add_argument('file', nargs="+", type=argparse.FileType("r"), \
    help='PanISa result files to merge')
command.add_argument('-v', '--version', action='version', \
    version='%(prog)s 0.0.2')


class URLParser(HTMLParser):
    urlres = ''
    def handle_starttag(self, tag, attrs):
        if tag == 'meta':
            for i, attr in enumerate(attrs):
                if i == 1:
                    self.urlres=str(attr[1]).lstrip("2;URL=")

class BlastParser(HTMLParser):
    ispre= False
    result = ""
    def handle_starttag(self, tag, attrs):
        if tag == "pre":
            self.ispre=True
    def handle_endtag(self, tag):
        if tag == "pre":
            self.ispre=False
    def handle_data(self, data):
        if self.ispre:
            self.result += data + "\n"

                    
def read_panisa(files, header, remove):
    mergedPanISa = []
    for fi in files: 
        first=fi.readline()
        for line in fi:
            tab = line.rstrip("\n").split("\t") 
            if len(tab) != 9:
                raise Exception("Not a panISa output or incorrect\n"+line)
            result = {k:v for k,v in zip(header,tab)}
            result["sample"] = fi.name.split("/")[-1].rstrip(remove)
            result["chrompos"] = result.get("chrom")+"_"+result.get("lpos")
            mergedPanISa.append(result)
    return mergedPanISa

def get_irl_irr(mergedPanISa, length):
    seqs = {}
    for v in mergedPanISa:
        name = v.get("sample")    
        irl = v.get("irl")[:length]
        irr = v.get("irr")[-length:]
        while len(irr) < length:
            irr = irr+"N"
        while len(irl) < length:
            irl = "N"+irl
        chroPos = v.get("chrompos")
        currentSeq = irl+irr
        if chroPos not in seqs:
            seqs[chroPos] = currentSeq
        elif currentSeq.count("N") < seqs.get(chroPos).count("N"):
            seqs[chroPos] = currentSeq
    return seqs


def search_ISfinder(potential_sequence):
    ##post data
    r = requests.post('https://www-is.biotoul.fr/blast/ncbiIS.php', \
                      data = {'title' : 'test', 'seq' : potential_sequence, 'seqfile' \
                              : '', 'database' : 'ISfindernt', 'prog' : 'blastn', 'blast' \
                              : 'ok', 'alignment' : '7', 'wordsize' : '11','expect' : '10.0' \
                              , 'gapcost' : '5 2'})
    ##get url result
    parser = URLParser()
    parser.feed(r.text)

    ##downoad blast result
    o = requests.get('https://www-is.biotoul.fr/blast/' + parser.urlres)    
    parser = BlastParser()
    parser.feed(o.text)
    return parser.result

def filter_blast(blast_result, identity, evalue, length, alignment):
    blasts = []
    column_name = ["Query", "Subject_ID", "%identity", "alignment_length", "mismatches", \
                   "gap_opens", "q.start", "q.end", "s.start", "s.end", "evalue", "bit_score"]
    for line in blast_result.rstrip("\n").split("\n"):
        if line.startswith('#'):
            continue
        blast = line.split("\t")
        if len(blast) != 12:
            sys.stderr.write(line+"\n")
        else:
            resultBlast = {k:v for k,v in zip(column_name, blast)}
            res_evalue = float(resultBlast.get("evalue"))
            res_identity = float(resultBlast.get("%identity"))
            res_alignment = int(resultBlast.get("q.end")) - int(resultBlast.get("q.start"))
            if res_evalue <= evalue and res_identity >= identity and \
               res_alignment >= (alignment*length)/100:
                blasts.append(line)
    return blasts

def write_result(output, mergedPanISa, length, resultISFinder):
    header = 'Sample\tChromosome\tStart_Position\tStop_Position\tPotential_sequence\tPotential_IS\tAlignment\n'
    output.write(header)
    for v in mergedPanISa:
        towrite = [v.get("sample")]
        towrite.append(v.get("chrom"))
        towrite.append(v.get("rpos"))
        towrite.append(v.get("lpos"))
        sequence = seqs.get(v.get("chrompos"))[:length] \
            + "-" + seqs.get(v.get("chrompos"))[-length:]
        towrite.append(sequence)
        isfinder = resultISFinder.get(v.get("chrompos"))
        if len(isfinder) == 0:
            towrite.append("No identity")
            towrite.append("")
        elif len(isfinder) == 1:
            IS = isfinder[0].split("\t")[1]
            towrite.append(IS)
            towrite.append("one side")
        else:
            IS = isfinder[0].split("\t")[1]
            if IS == isfinder[1].split("\t")[1] :
                towrite.append(IS)
                towrite.append("two side")
            else:
                towrite.append(IS)
                towrite.append("one side")
        output.write("\t".join(towrite) + "\n")
    

    
if __name__=='__main__':
    """Performed job on execution script"""
    args = command.parse_args()

    ##read panISa result and associate file name to sample
    header=["chrom", "lpos", "lclipped", 'dr', "rpos", "rclipped", "ir", "irl", "irr"]    
    mergedPanISa = read_panisa(args.file, header, args.remove)

    ##get irr-irl for each chromosome/position independly of the sample
    seqs = get_irl_irr(mergedPanISa, args.length)        

    ##search ISfinder and filter result
    resultISFinder = {}    
    for chrompos, potential_sequence in seqs.items():
        ##search homology in ISFinder
        blast_output = search_ISfinder(potential_sequence)

        ##get filter blast results:
        resultISFinder[chrompos] = filter_blast(blast_output, args.identity, args.evalue, \
                                                args.length, args.alignment)

    ##write result
    write_result(args.output, mergedPanISa, args.length, resultISFinder)




#!/usr/bin/python
# -*- coding: utf-8 -*-

##Copyright (c) 2017 Benoit Valot and Panisa Treepong
##benoit.valot@univ-fcomte.fr
##UMR 6249 Chrono-Environnement, Besançon, France
##Licence GPL

"""List of global variable setup"""

cigarclip = set([4,5])
